/*********************************************
Project :    SPS Betriebssystem auf STK200
Version :    1
Date    : 20.01.2004
Author  : Schweinester Josef
Company : ETS
Comments: 

Chip type           : AT90S2313
Clock frequency     : 4,000000 MHz
Memory model        : Tiny
External SRAM size  : 0
Data Stack size     : 32
*********************************************/

/*********************************************
	U P D A T E
	Date		: 14/10/2006
	Author		: Patrick Schöberl
	Chip type	: ATMELmega8
	Clock frequency	: 8 MHz
	
	New		: Ported to AVR-GCC
			: simple Makefile
			: UART-Functions
			: Counter
			: PWM
*********************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/delay.h>
#include <stdlib.h>

#define UBRR_BAUD		((F_CPU/(16L*UART_BAUD_RATE))-1)	

#define Tog_PA(Pos) 		PORTA ^=  (1 << Pos)		// toggle Portpin        
#define Tog_PB(Pos) 		PORTB ^=  (1 <<Pos)
#define Tog_PD(Pos) 		PORTD ^=  (1 << Pos)
#define Tog_Char(Byte,Pos)	Byte ^= (1<<Pos)		//Toggeld Charbit    
#define Set_Char(Byte,Pos)	Byte |= (1<<Pos)		//Bit setzen 
#define Clr_Char(Byte,Pos)	Byte &= ~(1<<Pos)		//Bit löchen    
 
//SPS  Macros

#define LE(Byte,Pos)		VK =  (Byte & (1 << Pos))    							//Eingangsbit in den VK
#define SA(Byte,Pos)		if (VK > 0)   Byte |= (1<< Pos); else  Byte &= ~(1<<Pos) ;	 		//VK in den Ausgang 
#define SB(Byte,Pos)		Byte |= (1<<Pos)	 							//Bit setzen ohne VK
#define SM(Byte,Pos)		if (VK > 0)   Byte |= (1<< Pos);      						//Merker setzen falls VK = 1
#define RM(Byte,Pos)		if (VK > 0)   Byte &= ~(1<<Pos);      						//Merker löchen falls VK = 1
#define L(Byte,Pos)		VK =  (Byte & (1 << Pos))    							//Merker in VK 
#define LN(Byte,Pos)		VK =  !(Byte & (1 << Pos))    							//Merker nicht  in VK	
#define R(Byte,Pos)		if  (VK > 0)  Byte &= ~(1<<Pos)  						//Merker resetten falls VK = 1
#define U(Byte,Pos)		VKH =  (Byte & (1 << Pos)) ;   if (VK  > 0  && VKH  > 0) VK = 1; else VK = 0;	//Verundung mit VK 
#define UN(Byte,Pos)		VKH =  !(Byte & (1 << Pos)) ;   if (VK  > 0  && VKH  > 0) VK = 1; else VK = 0;	//Verundung mit VK
#define O(Byte,Pos)		VKH =  (Byte & (1 << Pos)) ;   if (VK  > 0  || VKH  > 0) VK = 1; else VK = 0;	//Veroderung mit VK
#define ON(Byte,Pos)		VKH =  !(Byte & (1 << Pos)) ;   if (VK  > 0  || VKH  > 0) VK = 1; else VK = 0;	//Veroderung mit VK
#define CP(Byte,Byte1)		if (Byte == Byte1 ) VK = 1;else VK = 0;						//2 Bytes vergleichen
#define ST(Byte,Byte1)		if  (VK > 0)  Byte = Byte1;							//Timer laden mit Zeitwert
#define SW(Byte,Pos)		if (VK ==  0)  Clr_Char(Byte,Pos);   else  {VKH =  (Byte & (1 << Pos)); if (VKH  ==  0) Set_Char(Byte,Pos); else VK = 0;}
#define COM()			if (VK  >  0)   VK = 0 ;  else VK = 1 ;
#define TOG(Byte,Pos)		SW(TGBIT,Pos);  if (VK >  0)  Tog_Char(Byte,Pos) ;           			//Stromstoßchalter
#define ZV(Z)			if(Z<999) Z++;									//Zähle vorwärts (KoF)
#define ZR(Z)			if(Z>0) Z--;									//Zähle rückwärts (KoF)
#define PWM1(byte)		OCR1AL = byte									//PWM-Kanal 1 (KoF)
#define PWM2(byte)		OCR1BL = byte									//PWM-Kanal 2 (KoF)

int8_t  VK, VKH;			//Verknüpfungsspeicher und Hilfverknpfungsspeicher
uint8_t E0;				//Ein-Ausgäge , Merker,Sondermerker,Wischer,Toggler     
uint8_t A0;
uint8_t M0;
uint8_t SM_0;
uint8_t W0;
uint8_t TGBIT;
uint8_t TG ;	
//unsigned
uint8_t Temp1,Temp2;				//Temporäes Sicherungsbyte
uint16_t Z1 = 0;				//Zähler (KoF)	
uint8_t T1,T2,T3,T4;				//Timer    T1 +2 = 100ms, T3 +4 = 1 sec
int16_t Presc1,Presc2;				//Timervorteiler
void Reset (void);				//Resetroutine  
void SPS_Code (void);				//SPS Anwendercode
void uart_init(void);				//UART initialisieren (KoF)
void uart_send_char(uint8_t b);			//Zeichen senden (KoF)
void uart_send_string(int8_t *s);		//Zeichenkette senden(KoF)
void uart_send_int(uint16_t i);			//Integer senden (KoF)
void uart_send_float(float f);			//Float sendne (KoF)

//************************** Timer 0 overflow interrupt service routine ***alle 4 ms************************
SIGNAL(TIM0_OVF)
{
	Presc1++;
	if ( Presc1 >= 24)				//100 ms
	{
		Tog_Char (SM_0,0);			//Sondermerker
		if (T1  > 0) 
			T1--;
		if (T2  > 0) 
			T2--;
		Presc1 = 0;
	}
	Presc2++;
	if (Presc2 >= 245)			//1 sec.
	{
		Tog_Char (SM_0,1);		//Sondermerker     sec
		if (T3  > 0)
			T3--;
		if (T4  > 0)
			T4--;
		Presc2 = 0;
	}
}

void pwm_init(void)				//timer1 für nicht invertierende 8 bit pwms
{
	TCCR1A = (1<<COM1A1)|(1<<COM1B1)|(1<<WGM10);
	TCCR1B = (1<<CS10);
	TIMSK &= ~0x3c;
	PWM1(0);
	PWM2(0);
}
void uart_init(void)
{
	/* Baudrate einstellen ( Normaler Modus ), Baudrate wird in rnpower.h berechnet */
	UBRRH = (uint8_t) (UBRR_BAUD>>8);
	UBRRL = (uint8_t) UBRR_BAUD & 0x00FF;

	/* Aktivieren des Empf�gers, des Senders und des "Daten empfangen"-Interrupts */
	//UCSRB = (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);
	UCSRB = (1<<RXEN) | (1<<TXEN);

	/* Einstellen des Datenformats: 8 Datenbits, 1 Stoppbit */
	UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);
}
void uart_send_char(uint8_t b)
{
	while(!(UCSRA & (1<<UDRE)));
	UDR = b;
}
void uart_send_string(int8_t *s)
{
	while (*s)
	{
		uart_send_char(*s);
		s++;
	}
}
void uart_send_int(uint16_t i)
{
	int8_t s[7];
	itoa( i, s, 10); 	//10 = Dezimalsystem!
	uart_send_string(s);
}
void uart_send_float(float f)
{
	int8_t s[8];
	dtostrf(f,6,3,s);
	uart_send_string(s);
}
void Reset (void)
{
	DDRB=0xFF;
	PORTB = 0xFF;
 
	PORTD=0xff;
	DDRD=0x00;

	TCCR0=0x03;
	TCNT0=0x00;

	GIMSK=0x00;
	MCUCR=0x00;

	TIMSK=0x02;

	ACSR=0x80;
	sei();
}
int main(void)
{
	Reset ();				//Reset aufrufen
	uart_init();				//UART initialisieren
	pwm_init();				//PWM initialisieren
	while (1)
	{
		Temp1 =    PIND;		//Eingäge 2x einlesen und verunden  = entprellen	
		_delay_ms(30);
		Temp2 =    PIND;		//Eingäge einlesen und entprellen
		Temp1   = (Temp1   &   Temp2);	
		E0 = ~ Temp1;			//Eingange einlesen und entprellen
		SPS_Code();			//SPS Code einlesen
		PORTB = ~A0;			//Ausgäge setzen    ,neg logik     
		// delay_ms (200);
	}
	return 0;
}
//*********************************SPS Anwendercode******************************************* 
void SPS_Code (void)
{
	LN(M0,0);				//Einschaltverzöerung
	SW(W0,6);				//Timer nur einmal laden  
	ST(T1,30);				//Timer1 mit 30 laden (30 x 100ms)     
	CP(T1,0);				//Timer1 auf 0 abfragen
	SM(M0,0);				//Merker setzen falls Timer abgelaufen
	L(M0,0);				//Merker laden
	SA(A0,5);				//Ausgang 5 setzen,wenn Timer abgelaufen ist

	L(E0,0);				//Stromsto�chalter
	TOG(TG,0);
	L(TG,0);
	SA(A0,1);

	L(E0,1);				//Stromsto�chalter
	TOG(TG,1);
	L(TG,1);
	SA(A0,2);

	L(SM0,1);				//Blinkmerker 1000 ms
	SA(A0,3);				//Wechselbinklicht
	COM();					//VK negieren
	SA(A0,4); 
}
//***************************************************************************************
